{-# LANGUAGE FlexibleInstances #-}
import qualified Data.ByteString.Char8 as B
import qualified Data.ByteString.Builder as BB
import Data.Char (isSpace)
import Data.Foldable (foldl', toList)
import Data.List (scanl', mapAccumL, unfoldr)
import Control.Applicative (liftA2, liftA3)
import Control.Monad (replicateM, void)
import Control.Monad.Trans.State.Lazy (StateT (StateT, runStateT), evalStateT)
import System.IO (hFlush, stdout)
import qualified Data.Sequence as SQ

-- G-2

main :: IO ()
main = do
    [n, k, c] <- getInLine
    print $ resolve n k c

resolve :: Int -> Int -> Int -> Int
resolve n k c
    | c == 1    = 1
    | c == 2    = getModInt $ 2 ^ n
    | otherwise = getModInt $ acc0 + accsum
    where c' = modInt c
          initial = SQ.replicate (pred k) 0
          (acc0, _, accsum) = applyN (pred n) (step c') (c', initial, 0)

step :: ModInt -> (ModInt, SQ.Seq ModInt, ModInt) -> (ModInt, SQ.Seq ModInt, ModInt)
-- suppose i is the current position in the grid (1 <= i <= n),
-- and let 
--   * Ii be the interval (max 0 (i - k + 1), i),
--   * acc0i be the number of cases where all the colors in Ii are the same,
--   * accsi be the numbers of cases where there is a color different from the last color in Ii,
--     in the order of the last position j of such color (i + k - 1 <= j <= i - 1),
--     (acc0i is, so to speak, accsi ! Nothing) and
--   * accsumi be the sum of accsi. (only for the performance reason)
-- this step function calculates acc0(i + 1) and accs(i + 1) from acc0i and accsi.
-- let ci be the color at the position i in each case below.
-- from acc0i cases,
--   * c(i + 1) == ci: acc0i is added to acc0(i + 1),
--   * c(i + 1) /= ci: (c - 1) choices are possible, so acc0i * (c - 1) is added to accs(i + 1) ! i (acclst in the code below).
-- from each of accsi cases at j, when j > i + k - 1,
--   * c(i + 1) == ci: accsi ! j is added to accs(i + 1) ! j,
--   * c(i + 1) /= ci: since c(i + 1) should be the color at j, so accsi ! j is added to accs(i + 1) ! i (acclst);
-- when j == i + k - 1, since j is not included in I(i + 1),
--   * c(i + 1) == ci: all colors in I(i + 1) are the same, so accsi ! j (acc1 in the code below) is added to acc0(i + 1),
--   * c(i + 1) /= ci: (c - 1) choices are possible, so (accsi ! j) * (c - 1) is added to accs(i + 1) ! i (acclst).
step c' (acc0, accs, accsum) = (acc0', accs', accsum')
    where (acc1 SQ.:<| accs1) = accs
          acc0' = acc0 + acc1
          acclst = (c' - 1) * acc0 + (c' - 2) * acc1 + accsum
          accs' = accs1 SQ.|> acclst
          accsum' = accsum - acc1 + acclst

-- mod int

newtype ModInt = ModInt { getModInt :: Int }
    deriving (Eq, Ord, Show)
-- data constructor ModInt should not be used directly.
-- use modInt below instead.
-- the value inside ModInt is supposed to be ‘normalized’ by (`mod` modulus).

modInt :: Int -> ModInt
modInt = ModInt . (`mod` modulus)

modulus :: Int
modulus = 998244353

liftMod :: (Int -> Int) -> ModInt -> ModInt
liftMod op (ModInt x) = modInt $ op x

liftMod2 :: (Int -> Int -> Int) -> ModInt -> ModInt -> ModInt
liftMod2 op (ModInt x) (ModInt y) = modInt $ op x y

instance Num ModInt where
    (+) = liftMod2 (+)
    (*) = liftMod2 (*)
    abs = liftMod abs
    signum = liftMod signum
    fromInteger = modInt . fromInteger
    (-) = liftMod2 (-)

-- function

applyN :: Int -> (a -> a) -> a -> a
applyN n f x = iterate f x !! n

-- IO (Readable)

class Readable a where

    readBS :: B.ByteString -> Maybe (a, B.ByteString)
    readBS = runStateT readBSState

    readBSState :: StateT B.ByteString Maybe a
    readBSState = StateT readBS

instance Readable Int where
    readBS = B.readInt . dropSpace

instance Readable a => Readable [a] where
    readBS bs = Just (xs, B.empty)
        where xs = unfoldr readBS bs

discardLine :: IO ()
discardLine = void B.getLine

getInLine :: Readable a => IO a
getInLine = do
    Just x <- evalStateT readBSState <$> B.getLine
    return x

getInLines :: Readable a => Int -> IO [a]
getInLines n = replicateM n getInLine

dropSpace :: B.ByteString -> B.ByteString
dropSpace = B.dropWhile isSpace
