-- https://atcoder.jp/contests/abc278/submissions/36627576
{-# LANGUAGE BangPatterns, BinaryLiterals, CPP, DerivingStrategies        #-}
{-# LANGUAGE DerivingVia, FlexibleContexts, FlexibleInstances             #-}
{-# LANGUAGE GeneralizedNewtypeDeriving, KindSignatures, LambdaCase       #-}
{-# LANGUAGE MagicHash, MultiParamTypeClasses, MultiWayIf, NamedFieldPuns #-}
{-# LANGUAGE NumericUnderscores, OverloadedStrings, PatternSynonyms       #-}
{-# LANGUAGE RankNTypes, RecordWildCards, ScopedTypeVariables             #-}
{-# LANGUAGE StandaloneDeriving, TypeApplications          #-}
{-# LANGUAGE TypeFamilies, TypeInType, UnboxedTuples, ViewPatterns        #-}
{-# OPTIONS_GHC -O2 #-}

module Main where

import           Control.Applicative
import           Control.Exception
import           Control.Monad
import           Control.Monad.Primitive
import           Control.Monad.Reader
import           Control.Monad.ST
import           Control.Monad.State.Strict
import           Data.Bifunctor
import           Data.Bool
import qualified Data.ByteString                   as B
import qualified Data.ByteString.Builder           as B
import qualified Data.ByteString.Char8             as C
import qualified Data.ByteString.Internal          as B
import qualified Data.ByteString.Unsafe            as B
import           Data.Char
import qualified Data.Foldable                     as F
import           Data.Function
import           Data.Functor
import           Data.Functor.Identity
import qualified Data.IntMap.Strict                as IM
import qualified Data.IntSet                       as IS
import           Data.Kind
import qualified Data.List                         as L
import qualified Data.Map.Strict                   as M
import           Data.Maybe
import           Data.Monoid                       hiding (First (..),
                                                    Last (..))
import           Data.Ord
import           Data.Primitive
import           Data.Proxy
import           Data.Ratio
import           Data.Semigroup
import qualified Data.Set                          as S
import           Data.Tuple
import qualified Data.Vector                       as V
import qualified Data.Vector.Algorithms.Intro      as Intro
import qualified Data.Vector.Fusion.Bundle.Monadic as MB
import qualified Data.Vector.Fusion.Stream.Monadic as MS
import           Data.Vector.Fusion.Util
import qualified Data.Vector.Generic               as G
import qualified Data.Vector.Generic.Mutable       as GM
import qualified Data.Vector.Mutable               as VM
import qualified Data.Vector.Primitive             as P
import qualified Data.Vector.Primitive.Mutable     as PM
import qualified Data.Vector.Unboxed               as U
import qualified Data.Vector.Unboxed.Mutable       as UM
import           Debug.Trace
import           Foreign                           hiding (void)
import           GHC.Exts
import           GHC.TypeLits
import           GHC.Word
import           System.IO
import           Unsafe.Coerce

-- type IntMod = GF 1000000007 -- 998244353

main :: IO ()
main = runSolver (putBuilder . unlinesB B.intDec) $ do
    n <- int
    xs <- line $ uvectorN n int
    q <- int
    qs <- linesN q $ do
        uvectorN q queryP
    return $ solve n xs q qs

queryP :: PrimParser (Char, Int, Int)
queryP = do
    typ <- charSp
    case typ of
        '1' -> ('1',,-1) <$> uint
        '2' -> ('2',,) <$> uint1 <*> uint
        _-> ('3',,-1) <$> uint1

solve :: Int -> U.Vector Int -> Int -> U.Vector (Char, Int, Int) -> U.Vector Int
solve n xs q qs = runST $ do
    buf <- newBuffer q
    lastUpdate <- UM.replicate n ((-1) :: Int)
    diffs <- U.thaw xs
    U.ifoldM'_ (\prev@(!lastQ1, !lastX) qi -> \case
        ('1',x,_) -> pure (qi, x)
        ('2',i,x) -> do
            j <- UM.read lastUpdate i
            if j < lastQ1
            then do
                UM.write lastUpdate i qi
                UM.write diffs i (lastX + x)
            else do
                UM.write lastUpdate i qi
                UM.modify diffs (+x) i
            return prev
        (_,i,_) -> do
            j <- UM.read lastUpdate i
            if j < lastQ1
            then pushBack lastX buf
            else do
                res <- UM.read diffs i
                pushBack res buf
            return prev
        )
        (minBound, 0) qs
    unsafeFreezeBuffer buf


-------------------------------------------------------------------------------
-- My.Prelude
-------------------------------------------------------------------------------
rep :: (Monad m) => Int -> (Int -> m ()) -> m ()
rep n = flip MS.mapM_ (stream 0 n)
{-# INLINE rep #-}
rep1 :: (Monad m) => Int -> (Int -> m ()) -> m ()
rep1 n = flip MS.mapM_ (stream 1 (n + 1))
{-# INLINE rep1 #-}
rev :: (Monad m) => Int -> (Int -> m ()) -> m ()
rev n = flip MS.mapM_ (streamR 0 n)
{-# INLINE rev #-}
rev1 :: (Monad m) => Int -> (Int -> m ()) -> m ()
rev1 n = flip MS.mapM_ (streamR 1 (n + 1))
{-# INLINE rev1 #-}
stream :: (Monad m) => Int -> Int -> MS.Stream m Int
stream !l !r = MS.Stream step l where { step x | x < r = return $ MS.Yield x (x + 1) | otherwise = return MS.Done; {-# INLINE [0] step #-}}
{-# INLINE [1] stream #-}
streamR :: (Monad m) => Int -> Int -> MS.Stream m Int
streamR !l !r = MS.Stream step (r - 1) where { step x | x >= l = return $ MS.Yield x (x - 1) | otherwise = return MS.Done; {-# INLINE [0] step #-}}
{-# INLINE [1] streamR #-}
stream' :: (Monad m) => Int -> Int -> Int -> MS.Stream m Int
stream' !l !r !d = MS.Stream step l where { step x | x < r = return $ MS.Yield x (x + d) | otherwise = return MS.Done; {-# INLINE [0] step #-}}
{-# INLINE [1] stream' #-}
infixl 8 `shiftRL`, `unsafeShiftRL`
shiftRL :: Int -> Int -> Int
shiftRL = unsafeShiftRL
{-# INLINE shiftRL #-}
unsafeShiftRL :: Int -> Int -> Int
unsafeShiftRL (I# x#) (I# i#) = I# (uncheckedIShiftRL# x# i#)
{-# INLINE unsafeShiftRL #-}
unlinesB :: (G.Vector v a) => (a -> B.Builder) -> v a -> B.Builder
unlinesB f = G.foldr' ((<>) . (<> endlB) . f) mempty
unwordsB :: (G.Vector v a) => (a -> B.Builder) -> v a -> B.Builder
unwordsB f vec | G.null vec = mempty | otherwise = f (G.head vec) <> G.foldr' ((<>) . (B.char7 ' ' <>) . f) mempty (G.tail vec)
concatB :: (G.Vector v a) => (a -> B.Builder) -> v a -> B.Builder
concatB f = G.foldr ((<>) . f) mempty
matrixB :: (G.Vector v a) => Int -> Int -> (a -> B.Builder) -> v a -> B.Builder
matrixB h w f mat = F.foldMap ((<> endlB) . unwordsB f) [G.slice (i * w) w mat | i <- [0 .. h - 1]]
gridB :: (G.Vector v a) => Int -> Int -> (a -> B.Builder) -> v a -> B.Builder
gridB h w f mat = F.foldMap ((<> endlB) . concatB f) [G.slice (i * w) w mat | i <- [0 .. h - 1]]
sizedB :: (G.Vector v a) => (v a -> B.Builder) -> v a -> B.Builder
sizedB f vec = B.intDec (G.length vec) <> endlB <> f vec
yesnoB :: Bool -> B.Builder
yesnoB = bool (B.string7 "No") (B.string7 "Yes")
pairB :: (a -> B.Builder) -> (b -> B.Builder) -> (a, b) -> B.Builder
pairB f g (x, y) = f x <> B.char7 ' ' <> g y
showB :: (Show a) => a -> B.Builder
showB = B.string7 . show
showLnB :: (Show a) => a -> B.Builder
showLnB = B.string7 . flip shows "\n"
endlB :: B.Builder
endlB = B.char7 '\n'
{-# INLINE endlB #-}
putBuilder :: (MonadIO m) => B.Builder -> m ()
putBuilder = liftIO . B.hPutBuilder stdout
putBuilderLn :: (MonadIO m) => B.Builder -> m ()
putBuilderLn b = putBuilder b *> putBuilder (B.char7 '\n')
neighbor4 :: (Applicative f) => Int -> Int -> Int -> (Int -> f ()) -> f ()
neighbor4 h w xy f = when (x /= 0) (f $ xy - w) *> when (y /= 0) (f $ xy - 1) *> when (y /= w - 1) (f $ xy + 1) *> when (x /= h - 1) (f $ xy + w) where { (!x, !y) = quotRem xy w}
{-# INLINE neighbor4 #-}
binarySearchM :: (Monad m) => Int -> Int -> (Int -> m Bool) -> m Int
binarySearchM low0 high0 p = go low0 high0 where { go !low !high | high <= low = return high | otherwise = p mid >>= bool (go (mid + 1) high) (go low mid) where { mid = low + unsafeShiftRL (high - low) 1}}
{-# INLINE binarySearchM #-}
binarySearch :: Int -> Int -> (Int -> Bool) -> Int
binarySearch low high p = runIdentity $ binarySearchM low high (return . p)
{-# INLINE binarySearch #-}
radixSort :: U.Vector Int -> U.Vector Int
radixSort v0 = F.foldl' step v0 [0, 16, 32, 48] where { mask k x = unsafeShiftRL x k .&. 65535; step v k = U.create $ do { pos <- UM.unsafeNew 65537; UM.set pos 0; U.forM_ v $ \ x -> do { UM.unsafeModify pos (+ 1) (mask k x + 1)}; rep 65535 $ \ i -> do { fi <- UM.unsafeRead pos i; UM.unsafeModify pos (+ fi) (i + 1)}; res <- UM.unsafeNew $ U.length v; U.forM_ v $ \ x -> do { let { !masked = mask k x}; i <- UM.unsafeRead pos masked; UM.unsafeWrite pos masked $ i + 1; UM.unsafeWrite res i x}; return res}}
{-# INLINE radixSort #-}
encode32x2 :: Int -> Int -> Int
encode32x2 x y = unsafeShiftL x 32 .|. y
{-# INLINE encode32x2 #-}
decode32x2 :: Int -> (Int, Int)
decode32x2 xy = let { !x = unsafeShiftRL xy 32; !y = xy .&. 4294967295} in (x, y)
{-# INLINE decode32x2 #-}
uvectorN :: (U.Unbox a) => Int -> PrimParser a -> PrimParser (U.Vector a)
uvectorN = vectorN
{-# INLINE uvectorN #-}
bvectorN :: Int -> PrimParser a -> PrimParser (V.Vector a)
bvectorN = vectorN
{-# INLINE bvectorN #-}
vectorN :: (G.Vector v a) => Int -> PrimParser a -> PrimParser (v a)
vectorN n f = do { (e, o) <- viewPrimParser; pure $ G.unfoldrN n (pure . runPrimParser f e) o}
{-# INLINE vectorN #-}
runSolver :: (a -> IO ()) -> PrimParser a -> IO ()
runSolver = withInputHandle stdin
-------------------------------------------------------------------------------
-- Data.PrimParser
-------------------------------------------------------------------------------
newtype PrimParser a = PrimParser{runPrimParser# :: Addr# -> Addr# -> (# Addr#, a #)}
instance Functor PrimParser where { fmap f mx = PrimParser $ \ e p -> case runPrimParser# mx e p of { (# p', x #) -> (# p', f x #)}; {-# INLINE fmap #-}}
instance Applicative PrimParser where { pure x = PrimParser $ \ _ p -> (# p, x #); {-# INLINE pure #-}; mf <*> mx = PrimParser $ \ e p -> case runPrimParser# mf e p of { (# p', f #) -> case runPrimParser# mx e p' of { (# p'', x #) -> (# p'', f x #)}}; {-# INLINE (<*>) #-}; liftA2 f mx my = PrimParser $ \ e p -> case runPrimParser# mx e p of { (# p', x #) -> case runPrimParser# my e p' of { (# p'', y #) -> (# p'', f x y #)}}; {-# INLINE liftA2 #-}}
instance Monad PrimParser where { mx >>= f = PrimParser $ \ e p -> case runPrimParser# mx e p of { (# p', x #) -> runPrimParser# (f x) e p'}; {-# INLINE (>>=) #-}}
runPrimParser :: PrimParser a -> Ptr Word8 -> Ptr Word8 -> (a, Ptr Word8)
runPrimParser f (Ptr e#) (Ptr p#) = case runPrimParser# f e# p# of { (# p', x #) -> (x, Ptr p')}
{-# INLINE runPrimParser #-}
withByteString :: B.ByteString -> (a -> IO r) -> PrimParser a -> IO r
withByteString bs k f = case B.toForeignPtr bs of { (fp, o, I# len#) -> do { withForeignPtr (plusForeignPtr fp o) $ \ (Ptr p#) -> do { case runPrimParser# f (plusAddr# p# len#) p# of { (# _, x #) -> k x}}}}
unsafeWithByteString :: B.ByteString -> PrimParser a -> a
unsafeWithByteString bs f = B.accursedUnutterablePerformIO $ withByteString bs return f
withInputHandle :: Handle -> (a -> IO r) -> PrimParser a -> IO r
withInputHandle h k f = do { bs <- B.hGetContents h; withByteString bs k f}
withInputHandleUpToN :: Handle -> Int -> (a -> IO r) -> PrimParser a -> IO r
withInputHandleUpToN h bufSize k f = do { buf <- mallocForeignPtrBytes bufSize; withForeignPtr buf $ \ ibuf@(Ptr ibuf#) -> do { isize@(I# isize#) <- hGetBufNonBlocking h ibuf bufSize; pokeElemOff @Word8 (plusPtr ibuf isize) 0 10; case runPrimParser# f (plusAddr# ibuf# (isize# +# 1#)) ibuf# of { (# _, x #) -> k x}}}
viewPrimParser :: PrimParser (Ptr Word8, Ptr Word8)
viewPrimParser = PrimParser $ \ e p -> (# p, (Ptr e, Ptr p) #)
viewPrimParserAsByteString :: PrimParser B.ByteString
viewPrimParserAsByteString = PrimParser $ \ e p -> let { n = I# (minusAddr# e p); bs = B.unsafeCreate n $ \ dst -> B.memcpy dst (Ptr p) n} in (# p, bs #)
wordP# :: Addr# -> Word# -> (# Addr#, Word# #)
wordP# p# acc# = case indexWord8OffAddr# p# 0# of { c | isTrue# (geWord# c 48##) -> wordP# (plusAddr# p# 1#) (plusWord# (and# c 15##) (timesWord# 10## acc#)) | otherwise -> (# p#, acc# #)}
intP :: PrimParser Int
intP = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { 45## -> case wordP# (plusAddr# p 1#) 0## of { (# p', w #) -> (# p', I# (negateInt# (word2Int# w)) #)}; c -> case wordP# (plusAddr# p 1#) (and# c 15##) of { (# p', w #) -> (# p', I# (word2Int# w) #)}}
{-# INLINE intP #-}
wordP :: PrimParser Word
wordP = PrimParser $ \ _ p -> case wordP# p 0## of { (# p', w #) -> (# p', W# w #)}
{-# INLINE wordP #-}
doubleP :: PrimParser Double
doubleP = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { 45## -> case unsignedDoubleP# (plusAddr# p 1#) 0## of { (# p', d #) -> (# p', D# (negateDouble# d) #)}; c -> case unsignedDoubleP# (plusAddr# p 1#) (and# c 15##) of { (# p', d #) -> (# p', D# d #)}}
unsignedDoubleP# :: Addr# -> Word# -> (# Addr#, Double# #)
unsignedDoubleP# p w = case wordP# p w of { (# p', iw #) -> case indexWord8OffAddr# p' 0# of { 46## -> case wordP# (plusAddr# p' 1#) 0## of { (# p'', fw #) -> let { ipart = word2Double# iw; fpart = word2Double# fw /## indexDoubleOffAddr# pow10# (minusAddr# p'' p' -# 1#)} in (# p'', ipart +## fpart #)}; _ -> (# p', word2Double# iw #)} where { pow10# = "\NUL\NUL\NUL\NUL\NUL\NUL\240?\NUL\NUL\NUL\NUL\NUL\NUL$@\NUL\NUL\NUL\NUL\NUL\NULY@\NUL\NUL\NUL\NUL\NUL@\143@\NUL\NUL\NUL\NUL\NUL\136\195@\NUL\NUL\NUL\NUL\NULj\248@\NUL\NUL\NUL\NUL\128\132.A\NUL\NUL\NUL\NUL\208\DC2cA\NUL\NUL\NUL\NUL\132\215\151A\NUL\NUL\NUL\NULe\205\205A\NUL\NUL\NUL _\160\STXB\NUL\NUL\NUL\232vH7B\NUL\NUL\NUL\162\148\SUBmB\NUL\NUL@\229\156\&0\162B\NUL\NUL\144\RS\196\188\214B\NUL\NUL4&\245k\fC\NUL\128\224\&7y\195AC"#}}
{-# INLINE unsignedDoubleP# #-}
int :: PrimParser Int
int = PrimParser $ \ e p -> case runPrimParser# intP e p of { (# p', x #) -> (# plusAddr# p' 1#, x #)}
{-# INLINE int #-}
uint :: PrimParser Int
uint = PrimParser $ \ _ p -> case wordP# p 0## of { (# p', x #) -> (# plusAddr# p' 1#, I# (word2Int# x) #)}
{-# INLINE uint #-}
uint1 :: PrimParser Int
uint1 = PrimParser $ \ _ p -> case wordP# p 0## of { (# p', x #) -> (# plusAddr# p' 1#, I# (word2Int# x -# 1#) #)}
{-# INLINE uint1 #-}
double :: PrimParser Double
double = PrimParser $ \ e p -> case runPrimParser# doubleP e p of { (# p', x #) -> (# plusAddr# p' 1#, x #)}
{-# INLINE double #-}
char :: PrimParser Char
char = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { w8 -> (# plusAddr# p 1#, B.w2c (W8# w8) #)}
{-# INLINE char #-}
charSp :: PrimParser Char
charSp = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { w8 -> (# plusAddr# p 2#, B.w2c (W8# w8) #)}
{-# INLINE charSp #-}
digitC :: PrimParser Int
digitC = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { w -> (# plusAddr# p 1#, I# (word2Int# w -# 48#) #)}
{-# INLINE digitC #-}
lowerC :: PrimParser Int
lowerC = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { w -> (# plusAddr# p 1#, I# (word2Int# w -# 97#) #)}
{-# INLINE lowerC #-}
upperC :: PrimParser Int
upperC = PrimParser $ \ _ p -> case indexWord8OffAddr# p 0# of { w -> (# plusAddr# p 1#, I# (word2Int# w -# 65#) #)}
{-# INLINE upperC #-}
memchrP# :: Addr# -> Addr# -> Word8 -> Addr#
memchrP# e p w8 = let { !(Ptr pos) = B.accursedUnutterablePerformIO $ B.memchr (Ptr p) w8 (fromIntegral (I# (minusAddr# e p)))} in pos
memchrNthP# :: Int# -> Addr# -> Addr# -> Word8 -> Addr#
memchrNthP# n e ptr w8 = go n ptr where { go i p | isTrue# (i ># 1#) = go (i -# 1#) (plusAddr# (memchrP# e p w8) 1#) | otherwise = memchrP# e p w8}
byteStringTo :: Word8 -> PrimParser B.ByteString
byteStringTo w = PrimParser $ \ e p -> let { !end = memchrP# e p w; len = I# (minusAddr# end p); bs = B.unsafeCreate len $ \ dst -> B.memcpy dst (Ptr p) len} in (# plusAddr# end 1#, bs #)
{-# INLINE byteStringTo #-}
byteStringLn :: PrimParser B.ByteString
byteStringLn = byteStringTo 10
{-# INLINE byteStringLn #-}
byteStringSp :: PrimParser B.ByteString
byteStringSp = byteStringTo 32
{-# INLINE byteStringSp #-}
byteStringN :: Int -> PrimParser B.ByteString
byteStringN n@(I# n#) = PrimParser $ \ _ p -> let { bs = B.unsafeCreate n $ \ dst -> B.memcpy dst (Ptr p) n} in (# plusAddr# p n#, bs #)
{-# INLINE byteStringN #-}
line :: PrimParser a -> PrimParser a
line f = PrimParser $ \ e p -> case plusAddr# (memchrP# e p 10) 1# of { pos -> case runPrimParser# f pos p of { (# _, x #) -> (# pos, x #)}}
{-# INLINE line #-}
linesN :: Int -> PrimParser a -> PrimParser a
linesN (I# n#) f = PrimParser $ \ e p -> if isTrue# (n# ># 0#) then case plusAddr# (memchrNthP# n# e p 10) 1# of { pos -> case runPrimParser# f pos p of { (# _, x #) -> (# pos, x #)}} else case runPrimParser# f p p of { (# _, x #) -> (# p, x #)}
{-# INLINE linesN #-}
-------------------------------------------------------------------------------
-- Data.Buffer
-------------------------------------------------------------------------------
data Buffer s a = Buffer{bufferVars :: !(UM.MVector s Int), internalBuffer :: !(UM.MVector s a), internalBufferSize :: !Int}
_bufferFrontPos :: Int
_bufferFrontPos = 0
_bufferBackPos :: Int
_bufferBackPos = 1
newBuffer :: (U.Unbox a, PrimMonad m) => Int -> m (Buffer (PrimState m) a)
newBuffer n = Buffer <$> UM.replicate 2 0 <*> UM.unsafeNew n <*> pure n
type Stack s a = Buffer s a
newBufferAsStack :: (U.Unbox a, PrimMonad m) => Int -> m (Buffer (PrimState m) a)
newBufferAsStack n = Buffer <$> UM.replicate 2 0 <*> UM.unsafeNew n <*> pure n
type Queue s a = Buffer s a
newBufferAsQueue :: (U.Unbox a, PrimMonad m) => Int -> m (Buffer (PrimState m) a)
newBufferAsQueue n = Buffer <$> UM.replicate 2 0 <*> UM.unsafeNew n <*> pure n
type Deque s a = Buffer s a
newBufferAsDeque :: (U.Unbox a, PrimMonad m) => Int -> m (Buffer (PrimState m) a)
newBufferAsDeque n = Buffer <$> UM.replicate 2 n <*> UM.unsafeNew (2 * n) <*> pure (2 * n)
lengthBuffer :: (PrimMonad m) => Buffer (PrimState m) a -> m Int
lengthBuffer Buffer{bufferVars} = liftA2 (-) (UM.unsafeRead bufferVars _bufferBackPos) (UM.unsafeRead bufferVars _bufferFrontPos)
{-# INLINE lengthBuffer #-}
clearBuffer :: (PrimMonad m) => Buffer (PrimState m) a -> m ()
clearBuffer Buffer{bufferVars} = do { UM.unsafeWrite bufferVars _bufferFrontPos 0; UM.unsafeWrite bufferVars _bufferBackPos 0}
freezeBuffer :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (U.Vector a)
freezeBuffer Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; U.freeze $ UM.unsafeSlice f (b - f) internalBuffer}
unsafeFreezeBuffer :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (U.Vector a)
unsafeFreezeBuffer Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; U.unsafeFreeze $ UM.unsafeSlice f (b - f) internalBuffer}
freezeInternalBuffer :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (U.Vector a)
freezeInternalBuffer Buffer{bufferVars, internalBuffer} = do { b <- UM.unsafeRead bufferVars _bufferBackPos; U.freeze $ UM.unsafeSlice 0 b internalBuffer}
unsafeFreezeInternalBuffer :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (U.Vector a)
unsafeFreezeInternalBuffer Buffer{bufferVars, internalBuffer} = do { b <- UM.unsafeRead bufferVars _bufferBackPos; U.unsafeFreeze $ UM.unsafeSlice 0 b internalBuffer}
popFront :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (Maybe a)
popFront Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; if f < b then do { UM.unsafeWrite bufferVars _bufferFrontPos (f + 1); pure <$> UM.unsafeRead internalBuffer f} else return Nothing}
{-# INLINE popFront #-}
viewFront :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (Maybe a)
viewFront Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; if f < b then pure <$> UM.unsafeRead internalBuffer f else return Nothing}
{-# INLINE viewFront #-}
popBack :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (Maybe a)
popBack Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; if f < b then do { UM.unsafeWrite bufferVars _bufferBackPos (b - 1); pure <$> UM.unsafeRead internalBuffer (b - 1)} else return Nothing}
{-# INLINE popBack #-}
viewBack :: (U.Unbox a, PrimMonad m) => Buffer (PrimState m) a -> m (Maybe a)
viewBack Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; b <- UM.unsafeRead bufferVars _bufferBackPos; if f < b then pure <$> UM.unsafeRead internalBuffer (b - 1) else return Nothing}
{-# INLINE viewBack #-}
pushFront :: (U.Unbox a, PrimMonad m) => a -> Buffer (PrimState m) a -> m ()
pushFront x Buffer{bufferVars, internalBuffer} = do { f <- UM.unsafeRead bufferVars _bufferFrontPos; UM.unsafeWrite bufferVars _bufferFrontPos (f - 1); assert (f > 0) $ do { UM.unsafeWrite internalBuffer (f - 1) x}}
{-# INLINE pushFront #-}
pushBack :: (U.Unbox a, PrimMonad m) => a -> Buffer (PrimState m) a -> m ()
pushBack x Buffer{bufferVars, internalBuffer, internalBufferSize} = do { b <- UM.unsafeRead bufferVars _bufferBackPos; UM.unsafeWrite bufferVars _bufferBackPos (b + 1); assert (b < internalBufferSize) $ do { UM.unsafeWrite internalBuffer b x}}
{-# INLINE pushBack #-}
pushFronts :: (U.Unbox a, PrimMonad m) => U.Vector a -> Buffer (PrimState m) a -> m ()
pushFronts vec Buffer{bufferVars, internalBuffer} = do { let { n = U.length vec}; f <- UM.unsafeRead bufferVars _bufferFrontPos; UM.unsafeWrite bufferVars _bufferFrontPos (f - n); assert (n <= f) $ do { U.unsafeCopy (UM.unsafeSlice (f - n) n internalBuffer) vec}}
{-# INLINE pushFronts #-}
pushBacks :: (U.Unbox a, PrimMonad m) => U.Vector a -> Buffer (PrimState m) a -> m ()
pushBacks vec Buffer{bufferVars, internalBuffer, internalBufferSize} = do { let { n = U.length vec}; b <- UM.unsafeRead bufferVars _bufferBackPos; UM.unsafeWrite bufferVars _bufferBackPos (b + n); assert (b + n - 1 < internalBufferSize) $ do { U.unsafeCopy (UM.unsafeSlice b n internalBuffer) vec}}
{-# INLINE pushBacks #-}
